#include "Arduino.h"
#include "TeensyStep.h"
//#include "ADNS2610.h"
#include <Encoder.h>

//Encoder variables
#define  A_PHASE 14 // green cable
#define  B_PHASE 2 // white cable

const int gear_ratio = 6;

//Encoder object
Encoder encoder(A_PHASE, B_PHASE);

float ppr = gear_ratio*2458; //2458 is the number of pulses per encoder revolution
float deg_pulse = 360/ppr;
float encoder_angle = 0;

//Stepper configuration
const int steps_rev = 25000*gear_ratio;//25000 steps/revolution (microstepping setting)

//Maximum angle values to reach
int max_angle_deg = 90;
int min_angle_deg = 0; 
int max_angle_step = (max_angle_deg*steps_rev)/360;
int min_angle_step = (min_angle_deg*steps_rev)/360;

const int omega_deg = 20; // Angular velocity in deg*s^-1
const int alpha_deg = 200; // Angular acceleration in deg*s^-2

const int omega_steps = (omega_deg*steps_rev)/360; // Angular velocity in steps*s^-1
const int alpha_steps = (alpha_deg*steps_rev)/360; // Angular acceleration in steps*s^-2

float  desired_position = 0;
float  actual_position = 0;

constexpr unsigned P_Interval = 10; // ms  
constexpr float kp = 0.01; // (P)roportional constant of the regulator needs to be adjusted (depends on speed and acceleration setting)


Stepper stepper(5, 6); // STEP at pin 5, DIR at pin 6
RotateControl controller;

int32_t Setpoint = 0;
bool state = 0;

int hallSensorPin = 4;

int DAC_out_0;
int DAC_out_1;


void Homing()
{
  while (digitalRead(hallSensorPin) == HIGH)
  {
    TurnCounterClockwise();
  }
  delay(1000);
}

void TurnCounterClockwise ()
{
   digitalWrite(6,1);
   digitalWrite(5,HIGH);
   delayMicroseconds(100);
   digitalWrite(5,LOW);
   delayMicroseconds(100);
}
void Proportional_control()
{
  static unsigned lastTick = 0;

  if (millis() - lastTick > P_Interval)
  {    
    float delta = (Setpoint - stepper.getPosition()) * (kp / P_Interval);  // This implements a simple P regulator (can be extended to a PID if necessary)
    float factor = std::max(-1.0f, std::min(1.0f, delta)); // limit to -1.0..1.0

    controller.overrideSpeed(factor); // set new speed
    lastTick = 0;
  }
}

void setup()
{

  pinMode(hallSensorPin, INPUT);
  
  analogWriteResolution(12);
  
  //Optical1.begin(); 

  Serial.begin(112500);

  Homing();
  
  stepper
      .setMaxSpeed(omega_steps) 
      .setAcceleration(alpha_steps); 

  controller.rotateAsync(stepper);
  controller.overrideSpeed(0); // start with stopped slide

  //Encoder variable to zero after homing
  encoder.write(0);

}

elapsedMillis outputTimer = 0;
//Timer between setpoints
elapsedMillis reSetpointTimer = 0;
//Timer for the Pulse before starting the rotation
elapsedMillis PulseTimer = 0;

void loop()
{

  Proportional_control();
  
  encoder_angle = float(encoder.read()*deg_pulse*-1);
  actual_position = stepper.getPosition()*0.0024; //0.015=360deg/150000step
  
  Serial.print(encoder_angle);
  Serial.print(" ");
//  Serial.println(millis());
//  Serial.print(encoder.read());
//  Serial.print("  ");
  Serial.println((actual_position*-1));

  if(reSetpointTimer > 10000)  // new random target every 20 seconds
  {
    reSetpointTimer = 0;

    //Change the state variable
    state = !state;
    
    if (state == 0)
    {
      Setpoint = max_angle_step;
    }
    if (state == 1)
    {
      Setpoint = min_angle_step;
    }
  }
  //DAC_out_1 = int(5.6875*1.9*(actual_position) );
  //DAC_out_0 = int(5.6875*desired_position );
}
